/*
 * Admin specific styling are prefixed with body.change-form.
 */

.yourlabs-autocomplete
{
    position: absolute;
    left: 0;
    z-index: 1000;
    display: none;
    float: left;
    min-width: 160px;
    padding: 5px 0;
    margin: 2px 0 0;
    list-style: none;
    background-color: #ffffff;
    border: 1px solid #ccc;
    border: 1px solid rgba(0, 0, 0, 0.2);
    -webkit-border-radius: 6px;
    -moz-border-radius: 6px;
    border-radius: 6px;
    -webkit-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
    -moz-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
    box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
    -webkit-background-clip: padding-box;
    -moz-background-clip: padding;
    background-clip: padding-box;
}

.yourlabs-autocomplete .hilight
{
    color: #ffffff !important;
    text-decoration: none;
    background-color: #0081c2;
    background-image: -moz-linear-gradient(top, #0088cc, #0077b3);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#0088cc), to(#0077b3));
    background-image: -webkit-linear-gradient(top, #0088cc, #0077b3);
    background-image: -o-linear-gradient(top, #0088cc, #0077b3);
    background-image: linear-gradient(to bottom, #0088cc, #0077b3);
    background-repeat: repeat-x;
    outline: 0;
    filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff0088cc', endColorstr='#ff0077b3', GradientType=0);
}

.yourlabs-autocomplete .choice, 
.yourlabs-autocomplete [data-value], 
.yourlabs-autocomplete em
{
    display: block;
    padding: 3px 20px;
    clear: both;
    font-weight: normal;
    line-height: 20px;
    color: #333333;
    white-space: nowrap;
    color: black;
    cursor: pointer;
}

/* 
 * You should use the "block" class because the "div" class is deprecated since
 * 2.0. However, the "div" class is still supported for backward compatibility.
 */
.block,
.div
{
    display: block;
}

.autocomplete-light-widget {
    display: inline-block;
    /* for the add-another image */
    vertical-align: middle;
}

.autocomplete-light-widget input,
.autocomplete-light-widget input:focus {
    margin: 0;
    outline: 0;
    width: 230px;
}

.autocomplete-light-widget {
    /* "Position" the widget */
    position: relative;
}

input.autocomplete {
    position: relative;
    vertical-align: top;
}

input.autocomplete.xhr-pending {
    background-image: url('xhr-pending.gif');
    background-repeat: no-repeat;
    background-position: right;
}

.autocomplete-light-widget.yourlabs-autocomplete {
    position: absolute;
    top: 100%;
    left: 0;
    z-index: 100;
}

.autocomplete-light-widget .deck [data-value],
.autocomplete-light-widget .deck .choice
{
    list-style-type: none;
    display: inline-block;
    padding-top: 3px;
    padding-right: 3px;
}

.autocomplete-light-widget .deck [data-value] .remove
{
    background-image: url('delete.png');
    color: #fff;
    cursor: pointer;
    text-indent: -999px;
    height: 16px;
    width: 16px;
    display: inline-block;
    background-position-y: -1px;
    background-repeat: no-repeat;
}

.autocomplete-light-widget .deck [data-value] .remove:hover
{
    background-position: -15px 0;
}

.yourlabs-autocomplete .choice-detail, 
.yourlabs-autocomplete .choice-update 
{
    display: none;
}

/* grappelli specific */
fieldset.grp-module .grp-row {
    overflow: visible !important;
}

/* credit: http://www.csslab.cl/2008/01/30/ventana-modal-solo-con-css/ */
#yourlabs_overlay {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background: #000;
    z-index:1001;
    opacity:.75;
    -moz-opacity: 0.75;
    filter: alpha(opacity=75);
}
